package com.gic.evaluate.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.dto.EvaluateDTO;
import com.gic.evaluate.dto.MemberReplyDTO;
import com.gic.evaluate.entity.TabBadEvaluateConfig;
import com.gic.evaluate.entity.TabEvaluate;
import com.gic.evaluate.qo.EvaluateQO;
import com.gic.evaluate.service.BadEvaluateConfigService;
import com.gic.evaluate.service.EvaluateOutApiService;
import com.gic.evaluate.service.EvaluateReplyService;
import com.gic.evaluate.service.EvaluateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-27 15:27
 */
@Service("evaluateOutApiService")
public class EvaluateOutApiServiceImpl implements EvaluateOutApiService {

    @Autowired
    private EvaluateService evaluateService;
    @Autowired
    private EvaluateReplyService evaluateReplyService;
    @Autowired
    private BadEvaluateConfigService badEvaluateConfigService;

    @Override
    public ServiceResponse<Void> saveEvaluate(EvaluateDTO evaluateDTO) {
        TabEvaluate evaluate = evaluateService.getEvaluateByOrderNumber(evaluateDTO.getEnterpriseId(), evaluateDTO.getOrderNumber());
        if (evaluate != null) {
            EnterpriseServiceResponse.failure(ErrorCode.DATA_EXISTS);
        }

        evaluateDTO.setReviewStatus(Constants.CLOSE);
        evaluateDTO.setTopHotStatus(Constants.CLOSE);
        evaluateDTO.setMaliciousStatus(Constants.CLOSE);
        evaluateDTO.setShowStatus(Constants.OPEN);
        evaluateDTO.setVisitStatus(0);
        evaluateDTO.setReplyStatus(0);
        TabBadEvaluateConfig badEvaluateConfig = badEvaluateConfigService.getBadEvaluateConfig(evaluateDTO.getEnterpriseId());
        Integer badEvaluate = isBadEvaluate(badEvaluateConfig, evaluateDTO.getGoodsScore(), evaluateDTO.getStoreScore(), evaluateDTO.getServiceScore());
        Integer goodEvaluate = isGoodEvaluate(badEvaluateConfig, evaluateDTO.getGoodsScore(), evaluateDTO.getStoreScore(), evaluateDTO.getServiceScore());
        evaluateDTO.setBadEvaluateStatus(badEvaluate);
        evaluateDTO.setGoodEvaluateStatus(goodEvaluate);

        evaluateService.saveEvaluate(evaluateDTO);
        return EnterpriseServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> saveReply(MemberReplyDTO memberReplyDTO) {
        // 保存追评
        int line = evaluateReplyService.saveMemberReplyEvaluate(memberReplyDTO);
        if (line > 0) {
            EvaluateDTO evaluateDTO = new EvaluateDTO();
            evaluateDTO.setEvaluateId(memberReplyDTO.getEvaluateId());
            evaluateDTO.setReplyStatus(Constants.OPEN);
            evaluateService.updateEvaluate(evaluateDTO);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<EvaluateDTO>> listEvaluateByStore(EvaluateQO evaluateQO) {
        // todo 看到图在写
        return null;
    }

    @Override
    public ServiceResponse<Page<EvaluateDTO>> listEvaluateByMember(EvaluateQO evaluateQO) {
        // todo 看到图在写
        return null;
    }

    private Integer isBadEvaluate(TabBadEvaluateConfig badEvaluateConfig, Integer goodsScore, Integer storeScore, Integer serviceScore) {
        // 任何一项评价达到设定值
        if (Constants.OPEN.equals(badEvaluateConfig.getBadOnceStatus())) {
            if (matchConfigValue(badEvaluateConfig.getBadGoodsType(), badEvaluateConfig.getBadGoodsScore(), goodsScore)) {
                // 商品质量
                return Constants.OPEN;
            }
            if (matchConfigValue(badEvaluateConfig.getBadStoreType(), badEvaluateConfig.getBadStoreScore(), storeScore)) {
                // 门店服务
                return Constants.OPEN;
            }
            if (matchConfigValue(badEvaluateConfig.getBadServiceType(), badEvaluateConfig.getBadServiceScore(), serviceScore)) {
                // 服务能力
                return Constants.OPEN;
            }
        }
        // 三项评价均分
        if (Constants.OPEN.equals(badEvaluateConfig.getBadTotalStatus())) {
            if (matchConfigValue(badEvaluateConfig.getBadStoreType(), badEvaluateConfig.getBadStoreScore() * 3, goodsScore + storeScore + serviceScore)) {
                return Constants.OPEN;
            }
        }
        return Constants.CLOSE;
    }

    private Integer isGoodEvaluate(TabBadEvaluateConfig badEvaluateConfig, Integer goodsScore, Integer storeScore, Integer serviceScore) {
        // 任何一项评价达到设定值
        if (Constants.OPEN.equals(badEvaluateConfig.getGoodStatus())) {
            // 三项评价均分
            if (matchConfigValue(badEvaluateConfig.getGoodTotalType(), badEvaluateConfig.getGoodTotalScore() * 3, goodsScore + storeScore + serviceScore)) {
                if (matchConfigValue(badEvaluateConfig.getGoodOnceType(), badEvaluateConfig.getGoodOnceScore(), goodsScore)) {
                    // 商品质量
                    return Constants.OPEN;
                }
                if (matchConfigValue(badEvaluateConfig.getGoodOnceType(), badEvaluateConfig.getGoodOnceScore(), storeScore)) {
                    // 门店服务
                    return Constants.OPEN;
                }
                if (matchConfigValue(badEvaluateConfig.getGoodOnceType(), badEvaluateConfig.getGoodOnceScore(), serviceScore)) {
                    // 服务能力
                    return Constants.OPEN;
                }
            }
        }
        return Constants.CLOSE;
    }

    private boolean matchConfigValue(Integer computeType, Integer computeScore, Integer score) {
        switch (computeType) {
            case 1:
                return score < computeScore;
            case 2:
                return score <= computeScore;
            case 3:
                return score.equals(computeScore);
            case 4:
                return score >= computeScore;
            case 5:
                return score > computeScore;
            default:
                break;
        }
        return false;
    }
}
